/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.news.service.impl;

import com.alibaba.fastjson2.util.DateUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unis.config.NewsUrl;
import com.unis.news.entity.BizSinQinghaiNews;
import com.unis.news.mapper.BizSinQinghaiNewsMapper;
import com.unis.news.service.BizSinQinghaiNewsService;
import org.htmlunit.BrowserVersion;
import org.htmlunit.WebClient;
import org.htmlunit.html.HtmlAnchor;
import org.htmlunit.html.HtmlPage;
import org.htmlunit.html.HtmlParagraph;
import org.htmlunit.html.HtmlSpan;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 要闻概况-青海要闻管理表Service接口实现类
 *
 * @author qingsong.han
 * @date  2024/05/21 15:34
 **/
@Service
public class BizSinQinghaiNewsServiceImpl extends ServiceImpl<BizSinQinghaiNewsMapper, BizSinQinghaiNews> implements BizSinQinghaiNewsService {
    @Resource
    private NewsUrl newsUrl;
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizSinQinghaiNews addParam) {
        // 在添加
        this.save(addParam);
    }

    /**
     * 使用定时任务抓取数据
     */
    @Override
    public void taskAction() {
        WebClient webClient = null;
        try {
             webClient = new WebClient(BrowserVersion.CHROME);
            // 配置 WebClient 的参数，例如 JavaScript 支持
            webClient.getOptions().setJavaScriptEnabled(false);
            webClient.getOptions().setCssEnabled(false);
            final HtmlPage page=webClient.getPage(newsUrl.getQinghai());    //从指定URL获取HtmlPage
            List<HtmlParagraph> divList=page.getByXPath("//p[@class='item']");
            int i = 0;
            List<BizSinQinghaiNews> list1 = new ArrayList<>();
            for (HtmlParagraph node : divList) {
                if (i==8){ //只抓取最近8条数
                    break;
                }
                HtmlSpan span = node.getFirstByXPath("span[@class='gray']"); //得到发布日期
                HtmlAnchor anchor = node.getFirstByXPath("a[@target='_blank']"); //得到发布标题与地址
                if (span == null || anchor == null){
                    continue;
                }
                //先查询是否有相同标题内容，存在跳过
//                QueryWrapper<BizSinQinghaiNews> queryWrapper = new QueryWrapper<>();
//                queryWrapper.lambda()
//                        .eq(BizSinQinghaiNews::getTitle,anchor.getAttribute("title"))
//                        .eq(BizSinQinghaiNews::getSource,1); //来源
//                List<BizSinQinghaiNews> list =  this.list(queryWrapper);
//                if (!list.isEmpty()){ //表示此内容已经存在
//                    continue;
//                }
                BizSinQinghaiNews bizSinQinghaiNews  = new BizSinQinghaiNews();
                //发布日期
                String[] string = span.getTextContent().split("]");
                Date publishDate = DateUtils.parseDate(string[0].replace("[",""),"yyyy/MM/dd");
                bizSinQinghaiNews.setCreateTime(new Date());
                bizSinQinghaiNews.setUpdateTime(new Date());
                bizSinQinghaiNews.setPublishDate(publishDate);
                bizSinQinghaiNews.setTitle(anchor.getAttribute("title").trim());
                bizSinQinghaiNews.setRedirectUrl(anchor.getAttribute("href").trim());
                bizSinQinghaiNews.setIsRedirect(1);
                bizSinQinghaiNews.setSource(1); //来源
                bizSinQinghaiNews.setStatus(2); //默认使用提交状态
                list1.add(bizSinQinghaiNews);
                i++;
            }
            if (list1.size() >0) {
                //清理抓取数据，只保留今天抓取数据
                QueryWrapper<BizSinQinghaiNews> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda()
                        .eq(BizSinQinghaiNews::getSource, 1); //来源

                this.remove(queryWrapper);
                this.saveOrUpdateBatch(list1);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            webClient.close();    //关闭客户端
        }
    }

}
