/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.news.service.impl;

import com.alibaba.fastjson2.util.DateUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unis.config.NewsUrl;
import com.unis.news.entity.BizSinQinghaiNews;
import com.unis.news.entity.BizSinStateCouncilNotice;
import com.unis.news.mapper.BizSinStateCouncilNoticeMapper;
import com.unis.news.service.BizSinStateCouncilNoticeService;
import org.htmlunit.BrowserVersion;
import org.htmlunit.WebClient;
import org.htmlunit.html.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 要闻概况-国务院公告管理表Service接口实现类
 *
 * @author qingsong.han
 * @date  2024/07/23 10:18
 **/
@Service
public class BizSinStateCouncilNoticeServiceImpl extends ServiceImpl<BizSinStateCouncilNoticeMapper, BizSinStateCouncilNotice> implements BizSinStateCouncilNoticeService {
    @Resource
    private NewsUrl newsUrl;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizSinStateCouncilNotice addParam) {
        // 在添加
        this.save(addParam);
    }

    /**
     * 使用定时任务抓取数据
     */
    @Override
    public void taskAction() {
        WebClient webClient = null;
        try {
            webClient = new WebClient(BrowserVersion.CHROME);
            // 配置 WebClient 的参数，例如 JavaScript 支持
            webClient.getOptions().setJavaScriptEnabled(false);
            webClient.getOptions().setCssEnabled(false);
            final HtmlPage page=webClient.getPage(newsUrl.getStateCouncil());    //从指定URL获取HtmlPage

            List<HtmlHeading4> h4List=page.getByXPath("//h4");
            int i = 0;
            List<BizSinStateCouncilNotice> list1 = new ArrayList<>();
            for (HtmlHeading4 node : h4List) {
                if (i==8){ //只抓取最近8条数
                    break;
                }
                    HtmlSpan span = node.getFirstByXPath("span[@class='date']"); //得到发布日期
                    HtmlAnchor anchor = node.getFirstByXPath("a[@target='_blank']"); //得到发布标题与地址
                    if (span == null || anchor == null){
                        continue;
                    }

                String title = anchor.getTextContent().trim();
                //先查询是否有相同标题内容，存在跳过
//                    QueryWrapper<BizSinStateCouncilNotice> queryWrapper = new QueryWrapper<>();
//                    queryWrapper.lambda()
//                            .eq(BizSinStateCouncilNotice::getTitle,title)
//                            .eq(BizSinStateCouncilNotice::getSource,1); //来源
//                    List<BizSinStateCouncilNotice> list =  this.list(queryWrapper);
//                    if (!list.isEmpty()){ //表示此内容已经存在
//                        continue;
//                    }
                    BizSinStateCouncilNotice notice  = new BizSinStateCouncilNotice();
                    //发布日期
                    Date publishDate = DateUtils.parseDate(span.getTextContent().trim(),"yyyy-MM-dd");
                    notice.setCreateTime(new Date());
                    notice.setUpdateTime(new Date());
                    notice.setTitle(title);
                    notice.setPublishDate(publishDate);
                    notice.setRedirectUrl(anchor.getAttribute("href").trim());
                    notice.setIsRedirect(1);
                    notice.setSource(1); //来源
                    notice.setStatus(2); //默认使用提交状态
                    list1.add(notice);
                    i++;
            }
            //清理抓取数据，只保留今天抓取数据
            if (list1.size() >0){
                QueryWrapper<BizSinStateCouncilNotice> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda()
                        .eq(BizSinStateCouncilNotice::getSource,1); //来源
                this.remove(queryWrapper);
                this.saveOrUpdateBatch(list1);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }finally {
            webClient.close();    //关闭客户端
     }

    }
}
